//## FRAPo_LEICA_v01 ##
// reads a LIF file and processes all included series to the FRAP analysis, when timepolints >1
// requires user interaction
// requires Stackreg
// limited to one color so far
// manual cell outline, when autothrehold fails, especially when chromocenters are in the nucelus 
// 2020-12-04 Alex Rapp
// version 01
// version 02 added manualCheck for reading the results of the cell segmentation; changed stackReg to include the blank again

//########################
macro "FRAP-part1-Leica-semibatch" 

	{	
	requires("1.44");
	path = File.openDialog("Select a File");
	//path = "/Users/alexrapp/Desktop/FRAP_Leica/201202_FRAP_wt.lif"
	dir = File.getParent(path);
  	name = File.getName(path);
	run("Bio-Formats Macro Extensions");
	Ext.setId(path); 
	Ext.getSeriesCount(seriesCount);
	dir2 = getDirectory("Choose the out Directory ");
	//dir2	=	"/Users/alexrapp/Desktop/FRAP_Leica/out/"
	// loop over the series included
	for (s=0; s<seriesCount; s++)
		{
		Ext.setSeries(s);
		Ext.getSizeT(sizeT);
		// check if it is time series
		if (sizeT>1)
			{
			manualCheck=0;
			Ext.getSeriesName(seriesName);
			print(seriesName);
			run("Bio-Formats Importer", "open=["+ path + "] " + "view=[Standard ImageJ]" + " stack_order=Default " + "series_"+d2s(s+1,0));
			imageName = substring(seriesName,indexOf(seriesName, "/FRAP ")+6);
			rename("cell");
			run("Set Scale...", "distance=1 known=1 pixel=1 unit=pixel");
			// image stabilisation
			run("StackReg ", "transformation=[Rigid Body]");
			// make a copy for the cell outline
			run("Duplicate...", "title=cellmask duplicate range=1-nSlices");
			getRawStatistics(nPixels,mean,min,max,std,histogram);
			setMinAndMax(min, max);
			// clear the ROIManager from last imagename
			roiManager("reset");

			// find the total cell
			setAutoThreshold("Otsu dark");
			getThreshold(lower, upper);
			setThreshold(lower, upper);
			run("Set Measurements...", "area mean redirect=None decimal=3");
			run("Analyze Particles...", "size=1000-Infinity circularity=0.00-1.00 show=Nothing display exclude clear include add stack");
			// check that only one cell is found
			if (nResults!=nSlices)
				{
				run("Clear Results");
				w_title="More than one cell was detected";
				w_message="Adjust threshold manualy, \n then press OK";
				run("Threshold...");
				waitForUser(w_title,w_message);
				getThreshold(lower, upper);
				setThreshold(lower, upper);
				run("Set Measurements...", "area mean redirect=None decimal=3");
				run("Analyze Particles...", "size=1000-Infinity circularity=0.00-1.00 show=Nothing display exclude clear include add stack");
				}
			if (nResults!=nSlices)
				{
				run("Clear Results");
				roiManager("reset");
				w_title="More than one cell was detected again.";
				w_message="Draw the cell outline by hand \n then press OK";
				setTool("freehand");
				waitForUser(w_title,w_message);
				roiManager("Add");
				roiManager("Multi Measure");
				manualCheck=1;
				close();
				}
			// collect the cell intensities
			cell_intensity = newArray(nResults);
			for (n=0; n<nResults; n++)
				{
				if(manualCheck==1)
					{
					cell_intensity[n]=getResult("Mean1",n);
					}
				else
					{	
					cell_intensity[n]=getResult("Mean",n);
					}
				}
			run("Clear Results");
			roiManager("Reset");
			// get the ROIS from the user
			selectWindow("cell");
			setTool("elliptical");
			makeOval(30, 30, 12, 12);
			w_title="Is the bleached Roi correct?";
			w_message="Check whether the bleach ROI is correct or move the roi, \n then press OK";
			waitForUser(w_title,w_message);
			roiManager("Add");
			roiManager("Select", 0);
			roiManager("Rename", "BleachRoi");
							
			// not bleached rois
			setTool("oval");
			w_title="Make the not bleached Roi";
			w_message="Make a circular selection inside the cell, \n then press OK";
			waitForUser(w_title,w_message);
							
			getBoundingRect(BRX, BRY, RWidth, RHeight);
			if (BRX==0)
				{
				waitForUser(w_title,w_message);
				getBoundingRect(BRX, BRY, RWidth, RHeight);
				}
			makeOval(BRX, BRY, RWidth, RHeight);
			roiManager("Add");
			roiManager("Select", 1);
			roiManager("Rename", "NotbleachRoi");
			
			//measure the background
			w_title="Make the not background Roi";
			w_message="Make a circular selection outside the cell, \n then press OK";
			waitForUser(w_title,w_message);
			
			getBoundingRect(BGRX, BGRY, RGWidth, RGHeight);
			if (BGRX ==0)
				{
				waitForUser(w_title,w_message);
				getBoundingRect(BGRX, BGRY, RGWidth, RGHeight);
				}
			makeOval(BGRX,  BGRY, RGWidth, RGHeight);
			
			roiManager("Add");
			roiManager("Select", 2);
			roiManager("Rename", "BackgroundRoi");
			roiManager("Deselect");
			selectWindow("cell");
			roiManager("Multi Measure");

			// collect data measurements
			bleach_intensity = newArray(nResults);
			notbleach_intensity = newArray(nResults);
			background_intensity = newArray(nResults);
			for (n=0; n<nSlices; n++)
				{
				bleach_intensity[n]=getResult("Mean(BleachRoi)",n);
				notbleach_intensity[n]=getResult("Mean(NotbleachRoi)",n);
				background_intensity[n]=getResult("Mean(BackgroundRoi)",n);
				}
			// make the overview of the measures ROIs
			selectWindow("cell");
			setSlice(1);
			run("Colors...", "foreground=white background=white selection=yellow");
			roiManager("Draw");
			saveAs("tiff",dir2+imageName+"-overview.tif");
			close();
			// get the frametiming
			Ext.getSizeT(sizeT);
			deltaT = newArray(sizeT);
			for (no=0; no<sizeT; no++) 
				{
				Ext.getPlaneTimingDeltaT(deltaT[no], no);
				}
			// write results table
			run("Clear Results");
			prebleachGreenBG=(background_intensity[1]+background_intensity[2]+background_intensity[3]+background_intensity[4])/4;
			prebleachGreenROI=(bleach_intensity[1]+bleach_intensity[2]+bleach_intensity[3]+bleach_intensity[4])/4;
			prebleachGreenCell=(cell_intensity[1]+cell_intensity[2]+cell_intensity[3]+cell_intensity[4])/4;
			prebleachGreen=(prebleachGreenROI-prebleachGreenBG)/(prebleachGreenCell-prebleachGreenBG);
			for (n=0; n<sizeT; n++)
				{
				setResult("Timepoints",n, deltaT[n]);
				setResult("Cell-green",n, cell_intensity[n]);
				setResult("BR-green",n, bleach_intensity[n]);
				setResult("NBR-green",n, notbleach_intensity[n]);
				setResult("BG-green",n, background_intensity[n]);
				setResult("Accum-green",n, ((bleach_intensity[n]-background_intensity[n])/(cell_intensity[n]-background_intensity[n]))/prebleachGreen);
				updateResults();
				}
			saveAs("Results", dir2+imageName+"-rawmeasure.xls");
							
			}
		}

		}
	}