/*
 * Macro template to process multiple images in a folder
 */

#@ File (label = "Input directory", style = "directory") input
#@ File (label = "Output directory", style = "directory") output
#@ String (label = "File suffix", value = ".tif") suffix

// See also Process_Folder.py for a version of this code
// in the Python scripting language.

processFolder(input);

// function to scan folders/subfolders/files to find files with correct suffix
function processFolder(input) {
	list = getFileList(input);
	list = Array.sort(list);
	for (i = 0; i < list.length; i++) {
		if(File.isDirectory(input + File.separator + list[i]))
			processFolder(input + File.separator + list[i]);
		if(endsWith(list[i], suffix))
			processFile(input, output, list[i]);
	}
}

function processFile(input, output, file) {
	for (i = 0; i < list.length; i++) {
	run("Bio-Formats Importer", "open= + input + File.separator + file[i] autoscale color_mode=Grayscale rois_import=[ROI manager] view=Hyperstack stack_order=XYCZT");
	//run("Brightness/Contrast...")
	resetMinAndMax();

run("Z Project...", "projection=[Max Intensity]");
//rename("Master");


//do {

	Master = getTitle();
	selectWindow(Master);
	dotIndex = indexOf(Master, "."); // rename without extension
    title_nuc = substring(Master, 0, dotIndex);	
	//saveAs("Tiff", dir + title_nuc + "_z" + slice2);
	run("Duplicate...", "duplicate"); // duplicate again for chromocenter segmentation
	rename("forMask");
	run("Tile");


// Part 1: Nucleus Segmentation
selectWindow("forMask");
	run("8-bit");
	Stack.setPosition(3, 1, 1)
	selectWindow("forMask"); // choose image for nucleus segmentation
	run("Median...", "radius=3 stack");
	setOption("BlackBackground", true);
	run("Make Binary", "method=Triangle background=Dark calculate black");
	run("Watershed", "stack");
	selectWindow("forMask");
setTool("wand");
	waitForUser("Choose Nucleus", "Choose Nucleus-of-Interest, then press 'OK'"); // important for images containing multiple nuclei
	roiManager("Add");


// Part 2.1: Create CC mask
selectWindow(Master);
run("Duplicate...", "duplicate"); // duplicate again for chromocenter segmentation
	rename("forCCmask");
	run("Tile");
selectWindow("forCCmask");
	Stack.setPosition(3, 1, 1)
	roiManager("Select", 0);
	run("Clear Outside", "stack"); // limit signals to nucleus of interest from Part 1
	title1=getTitle();//returns a string with the image title 
	run("Split Channels");
imageCalculator("Average create", "C3-" + title1, "C3-" + title1);
	title2=getTitle();
	run("Tile");
selectWindow("C1-" + title1);
	close();
selectWindow("C2-" + title1);
	close(); 
selectWindow("C3-" + title1);
	close();
  

// part 2.2: Measure CC
run("Select None");
run("Threshold...");
	setAutoThreshold("Triangle dark");
	//setThreshold(2555, 65535);
	waitForUser("set CC threshold and press OK");
run("Create Mask");
	rename("CCmask");
	run("Create Selection");
	run("Tile");
selectWindow(title2);
close();
selectWindow(Master);
run("Duplicate...", "duplicate"); // duplicate again for chromocenter segmentation
	rename("forCC");
Stack.setPosition(2, 1, 1)
run("Restore Selection");
run("Make Inverse");
run("Measure");
run("Select None");	
Stack.setPosition(3, 1, 1)
run("Restore Selection");
run("Make Inverse");
run("Measure");
run("Select None");	
Stack.setPosition(1, 1, 1)
run("Restore Selection");
run("Make Inverse");
run("Measure");
run("Select None");


// Part 3.1: Create Nucleoplasm mask
selectWindow(Master);
run("Duplicate...", "duplicate"); // duplicate again for chromocenter segmentation
	rename("forNucmask");
		roiManager("Select", 0);
		run("Clear Outside", "stack"); // limit signals to nucleus of interest from Part 1	
			rename("forNuc");
selectWindow("CCmask");
	run("Create Selection");
selectWindow("forNuc");
	run("Restore Selection");	
	run("Clear Outside", "stack");
	run("Select None");
	roiManager("Select", 0);
	Stack.setPosition(2, 1, 1);
	run("Measure");
	Stack.setPosition(3, 1, 1);
	run("Measure");
	Stack.setPosition(1, 1, 1);
	run("Measure");

saveAs("Results", output + Master + ".csv");
selectWindow("CCmask");
saveAs("PNG", output + Master + ".png");

run("Close All");
selectWindow("Results"); 
   run("Close"); 
selectWindow("ROI Manager"); 
run("Close");
selectWindow("Threshold"); 
run("Close"); 

}
	}
